(function () {
    'use strict';

    class AnimatorStateScript extends Laya.AnimatorStateScript {
        constructor() {
            super();
            this.name = null;
            this._text = null;
            this.callback = null;
        }
        get text() {
            return this._text;
        }
        set text(value) {
            this._text = value;
        }
        /**
         * 动画状态开始时执行。
         */
        onStateEnter() {

        }
        /**
         * 动画状态更新时执行。
         */
        onStateUpdate() {

        }
        /**
         * 动画状态退出时执行。
         */
        onStateExit() {
            // console.log("执行动画回调:", this.callback);
            if (this.callback != null) {
                this.callback();
                this.callback = null;
            }
        }
    }

    class Animator extends Laya.Script {

        constructor() {
            super();
            this.ani = null;
            this.showLog = false;
            this.name = null;
        }

        onEnable() {
            this.ani = this.owner.getComponent(Laya.Animator);
            this.ani.speed = 1;
        }

        play(aniName, playFinishCallback = null) {
            // console.log("（过渡融合动画）播放动画名：" + aniName);
            this.name = aniName;
            this.createASS(aniName, playFinishCallback);
            this.ani.speed = 1;
            this.ani.crossFade(aniName, 0.2, 0);
        }

        play1(aniName, playFinishCallback = null) {
            // console.log("（普通动画）播放动画名：" + aniName);
            this.name = aniName;
            this.createASS(aniName, playFinishCallback);
            this.ani.speed = 1;
            this.ani.play(aniName);
        }

        createASS(aniName, playFinishCallback) {
            var stateChildren = this.ani._controllerLayers[0]._states;
            for (var key in stateChildren) {
                if (stateChildren[key].name == aniName) {
                    var state = stateChildren[key];
                    if (state._scripts != null) {
                        state._scripts[0].callback = playFinishCallback;
                    } else {
                        var ass = state.addScript(AnimatorStateScript);
                        ass.name = aniName;
                        ass.callback = playFinishCallback;
                    }
                }
            }
        }

        pause() {
            this.ani.speed = 0;
        }

        continue() {
            this.ani.speed = 1;
        }

        setAniSpeed(speed = 1) {
            this.ani.speed = speed;
        }

        onUpdate() {

        }

        onDisable() {
        }
    }

    class script_move {
        static move(obj, dir, player = false, forw, radians) {
            // console.log("开始移动   " + obj);
            if (obj == null)
                return false;
            let old = new Laya.Vector3(obj.transform.position.x, 0.1, obj.transform.position.z);
            obj.transform.translate(dir);
            let scene = SceneManager.mainScene;
            if (scene == null)
                scene = SceneManager.mainScene;
            let _ray = new Laya.Ray(new Laya.Vector3(obj.transform.position.x, 0.1 + 100, obj.transform.position.z), new Laya.Vector3(0, -1, 0));
            let _outHitInfo = [];
            this._outHitInfo1 = [];
            this._outHitInfo2 = [];
            this._outHitInfo3 = [];
            scene.physicsSimulation.rayCastAll(_ray, _outHitInfo);
            let ret = null;
            if (_outHitInfo.length == 0) {
                obj.transform.position = old;
                // console.log("射线长度到是0了    " + old);
                return false;
            }
            else {
                let yy = -100;
                for (let i = 0; i < _outHitInfo.length; i++) {
                    let r = _outHitInfo[i];
                    if (r.succeeded && r.collider.owner.layer == 2 && r.point.y > yy) {
                        // console.log("获取坐标点");
                        r.point.y = 0.1;
                        ret = r;
                        yy = r.point.y;
                    }
                    else if (r.collider.owner.layer == 8) {
                        // console.log("停止移动！");
                        ret = null;
                        break;
                    }
                }
                if (ret) {
                    obj.transform.position = ret.point;
                    // console.log("射线返回true");
                    return true;
                }
                else {
                    obj.transform.position = old;
                    // console.log("玩家停止移动，待在原地");
                    return false;
                }
            }
            return false;
            //        Laya.Vector3

        }
    }

    class script_building extends Laya.Script3D {
        init(idx, type) {
            this.model = this.owner;
            this.model.name = "building" + idx;
            this.type = type;
            this.liveFlag = true;
            this.shakeFlag = false;
            this.animator = this.model.addComponent(Animator);
            this.animator.play("Idle");
            this.target = null;
            this.player = false;
            this.targetName = null;
            switch (this.type) {
                case "A":
                    this.Health = 300;
                    break;
                case "B":
                    this.Health = 200;
                    break;
                case "C":
                    this.Health = 50;
                    break;
                case "D":
                    this.Health = 50;
                    break;
            }
            this.model.getChildAt(0).transform.rotate(new Laya.Vector3(0, SceneManager.RandomInt(0, 3) * 90, 0), false, false);
            this.model.getChildAt(0).meshRenderer.material.castShadow = true;
            this.model.getChildAt(0).meshRenderer.material.receiveShadow = true;
        }

        onUpdate() {
            if (this.target != null && !this.liveFlag) {
                this.moveBuilding();
            }
        }

        onDamage(val, npc = false, npcJS) {
            // this.owner.meshRenderer.material.albedoColor = new Laya.Vector4(0, 0.0, 0.0, 1.0);
            if (!this.shakeFlag) {
                this.shakeFlag = true;
                this.animator.play("Shake");
            }
            if (!npc) {
                if (SceneManager.triggerList.length > 1) {
                    let dis = Laya.Vector3.distance(new Laya.Vector3(SceneManager.blackHole.transform.position.x, 0, SceneManager.blackHole.transform.position.z), new Laya.Vector3(this.owner.transform.position.x, 0, this.owner.transform.position.z));
                    for (var key in SceneManager.triggerList) {
                        if (SceneManager.triggerList[key].owner != null) {
                            let otherDis = Laya.Vector3.distance(new Laya.Vector3(SceneManager.blackHole.transform.position.x, 0, SceneManager.blackHole.transform.position.z), new Laya.Vector3(SceneManager.triggerList[key].owner.parent.transform.position.x, 0, SceneManager.triggerList[key].owner.parent.transform.position.z));
                            if (otherDis < dis) {
                                this.reSetBuilding();
                                return;
                            }
                        } else {
                            continue;
                        }
                    }
                }
                if (!this.liveFlag)
                    return;
                if (this.type == "A" && SceneManager.gameUI.playerLevel < 1.5)
                    return;
                if (this.type == "B" && SceneManager.gameUI.playerLevel < 1.5) {
                    val /= 2;
                }
                if ((this.type == "C" || this.type == "D") && SceneManager.gameUI.playerLevel > 1.5) {
                    val *= 2;
                }
                if (this.type == "B" && SceneManager.gameUI.playerLevel > 1.8) {
                    val *= 2;
                }
                this.target = SceneManager.blackHole;
                this.targetName = "player";
                this.Health -= val;
                if (this.Health <= 0) {
                    this.liveFlag = false;
                    if (SceneManager.triggerList.length == 1) {
                        SceneManager.triggerList = [];
                    } else {
                        for (var key in SceneManager.triggerList) {
                            if (SceneManager.triggerList[key].owner != null) {
                                if (SceneManager.triggerList[key].owner.parent.name == this.model.name) {
                                    SceneManager.triggerList.splice(key, 1);
                                    break;
                                }
                            } else {
                                SceneManager.triggerList.splice(key, 1);
                            }
                        }
                    }
                    this.player = true;
                    SceneManager.banMove = true;
                    Laya.timer.once(0.2 * 1000, this, function () {
                        SceneManager.banMove = false;
                    });
                    this.animator.play("Collapse", function () {
                        Laya.timer.once(1, this, function () {
                            SceneManager.PlaySound("xi");
                            this.target = null;
                            this.model.destroy();
                        });
                        SceneManager.gameUI.addExp(this.type);
                    }.bind(this));
                }
            } else {
                if (npcJS.buildingList.length > 1) {
                    let dis = Laya.Vector3.distance(new Laya.Vector3(npcJS.model.transform.position.x, 0, npcJS.model.transform.position.z), new Laya.Vector3(this.owner.transform.position.x, 0, this.owner.transform.position.z));
                    for (var key in npcJS.buildingList) {
                        if (npcJS.buildingList[key].owner != null) {
                            let otherDis = Laya.Vector3.distance(new Laya.Vector3(npcJS.model.transform.position.x, 0, npcJS.model.transform.position.z), new Laya.Vector3(npcJS.buildingList[key].owner.parent.transform.position.x, 0, npcJS.buildingList[key].owner.parent.transform.position.z));
                            if (otherDis < dis) {
                                this.reSetBuilding();
                                return;
                            }
                        } else {
                            continue;
                        }
                    }
                }
                if (!this.liveFlag)
                    return;
                // if (this.type == "A" && npcJS.npcLevel < 1.5)
                //     return;
                // if (this.type == "B" && npcJS.npcLevel < 1.5) {
                //     val /= 2;
                // }
                // if ((this.type == "C" || this.type == "D") && npcJS.npcLevel > 1.5) {
                //     val *= 2;
                // }
                // if (this.type == "B" && npcJS.npcLevel > 1.8) {
                //     val *= 2;
                // }
                this.target = npcJS;
                this.targetName = npcJS.model.name;
                this.Health -= val;
                if (this.Health <= 0) {
                    this.liveFlag = false;
                    if (npcJS.buildingList.length == 1) {
                        npcJS.buildingList = [];
                    } else {
                        for (var key in npcJS.buildingList) {
                            if (npcJS.buildingList[key].owner != null) {
                                if (npcJS.buildingList[key].owner.parent.name == this.model.name) {
                                    npcJS.buildingList.splice(key, 1);
                                    break;
                                }
                            } else {
                                npcJS.buildingList.splice(key, 1);
                            }
                        }
                    }
                    this.animator.play("Collapse", function () {
                        Laya.timer.once(1, this, function () {
                            this.target = null;
                            npcJS.buildingDestroy();
                            npcJS.addExp(this.type);
                            this.model.destroy();
                        });
                    }.bind(this));
                    // console.log("NPC: ", npcJS.buildingList);
                }
            }
        }

        lookAt() {
            if (this.player) {
                this.model.transform.lookAt(this.target.transform.position, new Laya.Vector3(0, 1, 0));
            } else {
                this.model.transform.lookAt(this.target.model.transform.position, new Laya.Vector3(0, 1, 0));
            }
            this.model.transform.rotate(new Laya.Vector3(0, 180, 0), false, false);
        }

        moveBuilding() {
            // console.log("移动建筑物");
            this.lookAt();
            let spd;
            let dis;
            if (this.player) {
                dis = Laya.Vector3.distance(new Laya.Vector3(SceneManager.blackHole.transform.position.x, 0, SceneManager.blackHole.transform.position.z), new Laya.Vector3(this.owner.transform.position.x, 0, this.owner.transform.position.z));
            } else {
                dis = Laya.Vector3.distance(new Laya.Vector3(this.target.model.transform.position.x, 0, this.target.model.transform.position.z), new Laya.Vector3(this.owner.transform.position.x, 0, this.owner.transform.position.z));
            }
            if (dis < 1) {
                spd = 1 * Laya.timer.delta / 1000;
            } else {
                spd = 1 * dis * Laya.timer.delta / 1000;
            }
            if (dis > 10) {
                dis = 10;
            }
            if (false == script_move.move(this.model, new Laya.Vector3(0, 0, spd))) {
                if (true == script_move.move(this.model, new Laya.Vector3(0, 0, 0.5))) {
                    this.model.transform.rotate(new Laya.Vector3(0, 90, 0), false, false);
                }
                else if (true == script_move.move(this.model, new Laya.Vector3(0, 0, -0.5))) {
                    this.model.transform.rotate(new Laya.Vector3(0, 90, 0), false, false);
                }
            }
        }

        reSetBuilding() {
            if (!this.liveFlag)
                return;
            this.shakeFlag = false;
            this.targetName = null;
            this.target = null;
            this.animator.play("Idle");
            switch (this.type) {
                case "A":
                    this.Health = 300;
                    break;
                case "B":
                    this.Health = 200;
                    break;
                case "C":
                    this.Health = 50;
                    break;
                case "D":
                    this.Health = 50;
                    break;
            }
        }
    }

    class NPCTriggerCollisionScript extends Laya.Script3D {
        constructor() {
            super();
            this.ownerJS = null;
        }
        onTriggerEnter(other) {
            // console.log("============进入触发器范围==============");
            if (other.owner.parent.name.indexOf("building") != -1) {
                if (other.owner.parent.getComponent(script_building).liveFlag) {
                    this.ownerJS.buildingList.push(other);
                }
            }

        }

        onTriggerStay(other) {
            // let name = other.owner.parent.name;
            // console.log("NPC" + this.ownerJS.stayTime);
            if (this.ownerJS.state == -1 || this.ownerJS.targetPos != null)
                return;
            let damage = 100 * Laya.timer.delta / 1000;
            if (other.owner.parent.name.indexOf("building") != -1) {
                if (this.ownerJS.stayTime > 500 && other.owner.parent.getComponent(script_building).type == "A") {
                    this.ownerJS.attackBuilding();
                    other.owner.parent.getComponent(script_building).onDamage(damage, true, this.ownerJS);
                } else if (this.ownerJS.stayTime > 400 && other.owner.parent.getComponent(script_building).type == "B") {
                    this.ownerJS.attackBuilding();
                    other.owner.parent.getComponent(script_building).onDamage(damage, true, this.ownerJS);
                } else if (this.ownerJS.stayTime > 150 && other.owner.parent.getComponent(script_building).type == "C") {
                    this.ownerJS.attackBuilding();
                    other.owner.parent.getComponent(script_building).onDamage(damage, true, this.ownerJS);
                } else if (this.ownerJS.stayTime > 100 && other.owner.parent.getComponent(script_building).type == "D") {
                    this.ownerJS.attackBuilding();
                    other.owner.parent.getComponent(script_building).onDamage(damage, true, this.ownerJS);
                } else {
                    if (other.owner.parent.getComponent(script_building).target == null || (other.owner.parent.getComponent(script_building).target != null && other.owner.parent.getComponent(script_building).targetName == this.ownerJS.model.name)) {
                        this.ownerJS.stayTime += Laya.timer.delta;
                    }
                }
            }

            if (other.owner.name.indexOf("NPC") != -1) {
                // let damage = 100 * Laya.timer.delta / 1000;
                if (other.owner.getComponent(script_npc) != null) {
                    other.owner.getComponent(script_npc).onHit(this.ownerJS);
                }
            }

        }

        onTriggerExit(other) {
            // console.log("============离开触发器范围==============");
            if (other.owner.parent.name.indexOf("building") != -1) {
                other.owner.parent.getComponent(script_building).reSetBuilding();
                if (this.ownerJS.buildingList.length == 1) {
                    this.ownerJS.buildingList = [];
                } else {
                    for (var key in this.ownerJS.buildingList) {
                        if (this.ownerJS.buildingList[key].owner != null) {
                            if (this.ownerJS.buildingList[key].owner.parent.name == other.owner.parent.name) {
                                this.ownerJS.buildingList.splice(key, 1);
                                break;
                            }
                        } else {
                            this.ownerJS.buildingList.splice(key, 1);
                        }
                    }
                }
                this.ownerJS.stayTime = 0;
                this.ownerJS.continueMove();
            }
            // console.log(this.ownerJS.buildingList);
        }
        onCollisionEnter(collision) {
        }
        onCollisionStay(collision) {
        }
        onCollisionExit(collision) {
            // console.log("============离开碰撞器范围==============");
        }
        checkAtObject(name) {
            // console.log(name);
            if (name.indexOf("npc") >= 0) {
                // console.log(name.indexOf("car"));
                let id = name.substr(name.lastIndexOf("_") + 1);
                return parseInt(id);
            }
            return -1;
        }
    }

    class script_npc extends Laya.Script3D {
        init(idx) {
            this.npcLevel = 1;
            this.npcScale = 1;
            this.model = this.owner;
            this.model.name = "NPC" + idx;
            this.nameStr = SceneManager.RandomName();
            this.spd = 6;
            this.idx = idx;
            this.randomRotate();
            this.setState(npc_state_move);
            let triJS = this.model.addComponent(NPCTriggerCollisionScript);
            triJS.ownerJS = this;
            this.buildingList = [];
            this.stayTime = 0;
            this.levelUPExp = 0;
            this.targetPos = null;
            this.chaseTime = 0;
            this.score = 0;
            this.showPiFu(SceneManager.RandomInt(0, 20));
            this.setBeginSize();
        }

        setBeginSize() {
            var num = SceneManager.RandomInt(1, 3);
            switch (num) {
                case 2:
                    this.npcLevel = 1.1;
                    break;
                case 3:
                    this.npcLevel = 1.2;
                    break;
            }
        }

        showPiFu(id) {
            for (var key in this.model._children) {
                if (key != id) {
                    this.model._children[key].active = false;
                }
            }
        }

        onUpdate() {
            if (this.state == npc_state_destroy || SceneManager.gameUI.gameOverFlag) {
                return;
            }
            switch (this.state) {
                case npc_state_move:
                    this.moveNPC(Laya.timer.delta);
                    break;
                case npc_state_chase:
                    this.update_chase(Laya.timer.delta);
                    break;
                case npc_state_stop:
                    this.update_stop();
                    break;
            }
            if (this.npcScale != this.npcLevel) {
                // console.log("改变黑洞大小");
                if (this.npcScale < this.npcLevel) {
                    this.npcScale += 0.01;
                } else {
                    this.npcScale = this.npcLevel;
                }
                this.model.transform.setWorldLossyScale(new Laya.Vector3(this.npcScale, this.npcScale, this.npcScale));
            }
        }

        update_chase(dt) {
            if (this.chaseTime <= 5000) {
                this.chaseTime += dt;
                this.changeTargetPlayer();
                this.lookAt(this.targetPos);
                let dis = Laya.Vector3.distance(new Laya.Vector3(SceneManager.blackHole.transform.position.x, 0, SceneManager.blackHole.transform.position.z), new Laya.Vector3(this.owner.transform.position.x, 0, this.owner.transform.position.z));
                let spd = this.spd * Laya.timer.delta / 1000;
                if (false == script_move.move(this.model, new Laya.Vector3(0, 0, spd))) {
                    if (true == script_move.move(this.model, new Laya.Vector3(0, 0, 0.5))) {
                        this.lookAt(this.targetPos);
                    }
                    else if (true == script_move.move(this.model, new Laya.Vector3(0, 0, -0.5))) {
                        this.lookAt(this.targetPos);
                    }
                }
                if (dis < 0.55) {
                    this.chaseTime = 5500;
                    this.score += 100;
                    SceneManager.gameUI.playerOnHit();
                }
            } else {
                this.targetPos = null;
                this.randomRotate();
                this.setState(npc_state_move);
            }
        }

        update_stop() {
            if (this.buildingList.length == 1) {
                if (this.buildingList[0].owner != null) {
                    if (!this.buildingList[0].owner.parent.getComponent(script_building).liveFlag) {
                        this.buildingList = [];
                        this.buildingDestroy();
                    }
                } else {
                    this.buildingList = [];
                    this.buildingDestroy();
                }
            }
        }

        attackBuilding() {
            if (this.state != npc_state_stop) {
                // console.log("NPC停止移动");
                this.setState(npc_state_stop);
                Laya.timer.clear(this, this.randomRotate);
            }
        }

        continueMove() {
            if (this.state == npc_state_destroy)
                return;
            if (this.state != npc_state_move) {
                // console.log("NPC继续移动");
                this.stayTime = 0;
                this.randomRotate();
                this.setState(npc_state_move);
            }
        }

        buildingDestroy() {
            // console.log("建筑消失，NPC继续移动");
            if (this.state == npc_state_destroy)
                return;
            this.stayTime = 0;
            this.randomRotate();
            this.setState(npc_state_move);
        }

        setState(state) {
            if (this.state == state)
                return;
            switch (state) {
                case npc_state_move:
                    break;
                case npc_state_death:
                    break;
                case npc_state_chase:
                    Laya.timer.clear(this, this.randomRotate);
                    this.changeTargetPlayer();
                    break;
                case npc_state_destroy:
                    let p = SceneManager.mainScene.getChildByName("Born" + SceneManager.RandomInt(1, 10));
                    this.model.transform.position = p.transform.position;
                    break;
            }
            this.state = state;
        }

        randomRotate() {
            let initlook = SceneManager.RandomInt(0, 360);
            let num = SceneManager.RandomInt(0, 100);
            if (num < 50) {
                this.jiaodu = 1;
            } else {
                this.jiaodu = -1;
            }
            this.model.transform.rotate(new Laya.Vector3(0, initlook * this.jiaodu, 0), false, false);
            Laya.timer.once(SceneManager.RandomInt(5, 10) * 1000, this, this.randomRotate);
        }

        moveNPC(dt) {
            if (this.chaseTime == 0) {
                if (this.findPlayer()) {
                    this.setState(npc_state_chase);
                    return;
                }
            } else {
                this.chaseTime -= dt;
                if (this.chaseTime <= 0) {
                    this.chaseTime = 0;
                }
            }
            let spd = this.spd * Laya.timer.delta / 1000;
            if (false == script_move.move(this.model, new Laya.Vector3(0, 0, spd))) {
                if (true == script_move.move(this.model, new Laya.Vector3(0, 0, 0.5))) {
                    this.model.transform.rotate(new Laya.Vector3(0, 90, 0), false, false);
                }
                else if (true == script_move.move(this.model, new Laya.Vector3(0, 0, -0.5))) {
                    this.model.transform.rotate(new Laya.Vector3(0, 90, 0), false, false);
                }
            }
        }

        onHit() {
            let dis = Laya.Vector3.distance(new Laya.Vector3(SceneManager.blackHole.transform.position.x, 0, SceneManager.blackHole.transform.position.z), new Laya.Vector3(this.owner.transform.position.x, 0, this.owner.transform.position.z));
            if (SceneManager.gameUI.playerLevel > this.npcLevel + 0.3 && dis <= 0.55) {
                SceneManager.gameUI.score += 100;
                SceneManager.showAddScore("+100");
                SceneManager.gameUI.killNPCNum++;
                SceneManager.gameUI.changeKillText();
                this.refreshRankList();
                Laya.timer.clear(this, this.randomRotate);
                this.model.transform.position = new Laya.Vector3(this.model.transform.position.x, 100, this.model.transform.position.z);
                this.buildingList = [];
                this.targetPos = null;
                this.setState(npc_state_destroy);
                Laya.timer.once(5 * 1000, this, function () {
                    // console.log("复活NPC");
                    this.setState(npc_state_move);
                    this.model.transform.position = new Laya.Vector3(this.model.transform.position.x, 0.1, this.model.transform.position.z);
                    this.stayTime = 0;
                    this.randomRotate();
                });
            }
        }

        onHitByNPC(npc) {
            let dis = Laya.Vector3.distance(new Laya.Vector3(npc.model.transform.position.x, 0, npc.model.transform.position.z), new Laya.Vector3(this.owner.transform.position.x, 0, this.owner.transform.position.z));
            if (npc.npcLevel > this.npcLevel + 0.3 && dis <= 0.55) {
                npc.score += 100;
                this.refreshRankList();
                Laya.timer.clear(this, this.randomRotate);
                this.model.transform.position = new Laya.Vector3(this.model.transform.position.x, 100, this.model.transform.position.z);
                this.buildingList = [];
                this.targetPos = null;
                this.setState(npc_state_destroy);
                Laya.timer.once(5 * 1000, this, function () {
                    // console.log("复活NPC");
                    this.setState(npc_state_move);
                    this.model.transform.position = new Laya.Vector3(this.model.transform.position.x, 0.1, this.model.transform.position.z);
                    this.stayTime = 0;
                    this.randomRotate();
                });
            }
        }

        findPlayer() {
            let dis = Laya.Vector3.distance(new Laya.Vector3(SceneManager.blackHole.transform.position.x, 0, SceneManager.blackHole.transform.position.z), new Laya.Vector3(this.owner.transform.position.x, 0, this.owner.transform.position.z));
            if (dis <= 5 && SceneManager.gameUI.playerLevel + 0.3 < this.npcLevel && !SceneManager.gameUI.deathFlag) {
                return true;
            }
            return false;
        }

        changeTargetPlayer() {
            this.targetPos = new Laya.Vector3(SceneManager.blackHole.transform.position.x, 0.1, SceneManager.blackHole.transform.position.z);
        }

        lookAt(targetPos) {
            this.model.transform.lookAt(targetPos, new Laya.Vector3(0, 1, 0));
            this.model.transform.rotate(new Laya.Vector3(0, 180, 0), false, false);
        }

        addExp(type) {
            switch (type) {
                case "A":
                    this.levelUPExp += 50;
                    this.score += 50;
                    if (this.levelUPExp >= 100) {
                        this.levelUPExp = 0;
                        this.levelUP();
                    }
                    this.refreshRankList();
                    break;
                case "B":
                    this.levelUPExp += 40;
                    this.score += 40;
                    if (this.levelUPExp >= 100) {
                        this.levelUPExp = 0;
                        this.levelUP();
                    }
                    this.refreshRankList();
                    break;
                case "C":
                    this.levelUPExp += 30;
                    this.score += 30;
                    if (this.levelUPExp >= 100) {
                        this.levelUPExp = 0;
                        this.levelUP();
                    }
                    this.refreshRankList();
                    break;
                case "D":
                    this.levelUPExp += 20;
                    this.score += 20;
                    if (this.levelUPExp >= 100) {
                        this.levelUPExp = 0;
                        this.levelUP();
                    }
                    this.refreshRankList();
                    break;
            }
        }

        levelUP() {
            if (this.npcLevel < 2) {
                this.npcLevel += 0.2;
            } else if (this.npcLevel < 3) {
                this.npcLevel += 0.18;
            } else {
                this.npcLevel += 0.15;
            }
            if (this.npcLevel > 4) {
                this.npcLevel = 4;
            }
        }

        refreshRankList() {
            let id;
            for (var key in SceneManager.rankList) {
                if (SceneManager.rankList[key] == this.idx)
                    id = key;
            }
            SceneManager.score[id] = this.score;
            let temp;
            for (let i = 0; i < 10; i++) {
                for (let j = i + 1; j < 10; j++) {
                    if (SceneManager.score[i] < SceneManager.score[j]) {
                        temp = SceneManager.score[i];
                        SceneManager.score[i] = SceneManager.score[j];
                        SceneManager.score[j] = temp;
                        temp = SceneManager.rankList[i];
                        SceneManager.rankList[i] = SceneManager.rankList[j];
                        SceneManager.rankList[j] = temp;
                    }
                }
            }
            SceneManager.gameUI.updateRankingList();
        }
    }
    var npc_state_move = 1;
    var npc_state_death = 2;
    var npc_state_chase = 3;
    var npc_state_stop = 4;
    var npc_state_destroy = -1;

    class SceneManager {
        static open(_ui) {
            switch (_ui) {
                case SceneManager.UI_MAIN:
                    Laya.Scene.open("main.scene");
                    SceneManager.gameUI = null;
                    break;
                case SceneManager.UI_GAME:
                    Laya.Scene.open("blackholegame.scene");
                    SceneManager.mainUI = null;
                    break;
            }
            SceneManager.curScene = _ui;
        }
        static PlayMusic(id) {
            var sn;
            switch (id) {
                case 1:
                    sn = "res/sound/bgm.wav";
                    console.log("播放进入主菜单的音乐");
                    break;
            }
            Laya.loader.load(sn, Laya.Handler.create(this, () => {
                Laya.SoundManager.playMusic(sn, 0);
            }));
        }
        static PlaySound(name, loop = 1) {
            var sn = "res/sound/" + name;
            var sc = null;
            sc = Laya.SoundManager.playSound(sn + ".wav", loop);
            return sc;
        }
        static StopSound(name) {
            if (name == null) {
                Laya.SoundManager.stopAll();
            }
            else {
                var sn = "res/sound/" + name;
                Laya.SoundManager.stopSound(sn + ".wav");
            }
        }
        static RandomInt(min, max) {
            return Math.round(Math.random() * (max - min) + min);
        }

        static showAddScore(str) {
            if (SceneManager.messageUI == null) {
                Laya.Scene.open("message.scene", false, [1, str]);
            }
            else {
                SceneManager.messageUI.ShowInfo(str);
            }
        }

        static backToMain() {
            Laya.timer.clearAll(SceneManager.gameUI);
            for (var key in SceneManager.NPCList) {
                Laya.timer.clearAll(SceneManager.NPCList[key].getComponent(script_npc));
            }
            SceneManager.mainScene.destroy();
            SceneManager.mainScene = null;
            SceneManager.clearData();
            SceneManager.rockerView.destroy();
            SceneManager.open(SceneManager.UI_MAIN);
        }

        static restartGame() {
            Laya.timer.clearAll(SceneManager.gameUI);
            for (var key in SceneManager.NPCList) {
                Laya.timer.clearAll(SceneManager.NPCList[key].getComponent(script_npc));
            }
            SceneManager.mainScene.destroy();
            SceneManager.mainScene = null;
            SceneManager.clearData();
            SceneManager.rockerView.destroy();
            Laya.timer.once(1, this, function () {
                Laya.Scene.open("loading.scene");
            });
        }

        static clearData() {
            SceneManager.blackHole = null;
            SceneManager.buildingsList = [];
            SceneManager.NPCList = [];
            SceneManager.triggerList = [];
            SceneManager.banMove = false;
            SceneManager.speedUPFlag = false;
            SceneManager.skinID = 1;
            SceneManager.score = [0, 0, 0, 0, 0, 0, 0, 0, 0, 0];
            SceneManager.rankList = [1, 2, 3, 4, 5, 6, 7, 8, 9, 10];
            SceneManager.nameid = [];
        }

        static RandomName() {
            let id = this.RandomInt(0, SceneManager.nameList.length - 1);
            if (SceneManager.nameid.length == 0) {
                SceneManager.nameid.push(id);
            }
            else {
                for (var key in SceneManager.nameid) {
                    if (SceneManager.nameid[key] == id) {
                        return this.RandomName();
                    }
                }
                SceneManager.nameid.push(id);
            }
            return SceneManager.nameList[id];
        }
    }
    SceneManager.UI_MAIN = 1;
    SceneManager.UI_GAME = 2;
    SceneManager.curScene = -1;
    SceneManager.loadingUI = null;
    SceneManager.mainUI = null;
    SceneManager.gameUI = null;
    SceneManager.pauseUI = null;
    SceneManager.mainScene = null;
    SceneManager.model = null;
    SceneManager.blackHole = null;
    SceneManager.buildingsList = [];
    SceneManager.NPCList = [];
    SceneManager.triggerList = [];
    SceneManager.banMove = false;
    SceneManager.speedUPFlag = false;
    SceneManager.skinID = 1;
    SceneManager.score = [0, 0, 0, 0, 0, 0, 0, 0, 0, 0];
    SceneManager.rankList = [1, 2, 3, 4, 5, 6, 7, 8, 9, 10];
    SceneManager.nameid = [];
    SceneManager.nameList = [
        "神度", "橘州", "一百昏的走位", "赢一把就睡觉", "亚瑟瑟瑟发抖",
        "她有对象先扶我", "别投我还能秀", "安全区扛把子", "一枪一个男朋友", "Ustinian", "Nuyoah",
        "花醉人", "我总是太早或太晚", "伯雾浓", "元气少女谢广坤", "代表月亮突突你", "宋友慈",
        "开心市民小张", "滕老鸦", "乔留水", "人间的面见一面少一面", "天边透亮的星", "怎么吃不饱",
        "机场霸主", "云希", "宋吟", "阎漪", "凤岚", "脸上有肉", "方璇", "快乐太难",
        "Gnomeshgh", "算了狗命要紧", "捞起月亮的渔民", "落日弥漫的橘", "我祝你想哭就哭",
        "月亮邮递员", "苍神光", "拥向宇宙吹来的风", "上月", "葬爱家族创始人",
        "读不出我任何欲望", "你压到我腿毛了", "宇宙与你相撞",
        "蓝樱", "朵姬", "Eliauk", "璎珞", "颜离", "Famiglistimo", "清沙", "婉儿", "素未谋面", "莫奈", "离奈",
        "我是哈批", "与夏夜相逢", "梦伴", "银河由此诞生", "星空不问赶路人", "眸敛", "星星碎掉溢出的光", "浪漫世界值得孤身", "暖心忆", "逆蝶", "多次婉拒彭于晏", "时光倒影", "绘梦", "寒烟冷", "浅暮流殇"

    ];

    class script_player {
        //移动
        static Move(dirX, dirY, radians) {
            let spd = script_player._speed * Laya.timer.delta / 1000;
            if (SceneManager.speedUPFlag) {
                spd *= 1.2;
            }
            if (false == script_move.move(SceneManager.blackHole, new Laya.Vector3(dirX * spd, 0, dirY * spd), true, true, radians)) {
                if (true == script_move.move(SceneManager.blackHole, new Laya.Vector3(spd * 0.5, 0, spd * 0.5), true, true, radians)) {
                }
                else if (true == script_move.move(SceneManager.blackHole, new Laya.Vector3(-spd * 0.5, 0, -spd * 0.5), true, true, radians)) {
                }
            }
        }
    }
    script_player._speed = 10;

    class TriggerCollisionScript extends Laya.Script3D {
        constructor() {
            super();
        }
        onTriggerEnter(other) {
            // console.log("============进入触发器范围==============");
            if (!SceneManager.gameUI.deathFlag) {
                if (other.owner.parent.name.indexOf("building") != -1) {
                    SceneManager.triggerList.push(other);
                }
            }

        }

        onTriggerStay(other) {
            // let name = other.owner.parent.name;
            // console.log(other.owner.name);
            if (!SceneManager.gameUI.deathFlag) {
                if (other.owner.parent.name.indexOf("building") != -1) {
                    let damage = 100 * Laya.timer.delta / 1000;
                    if (other.owner.parent.getComponent(script_building).target == null || (other.owner.parent.getComponent(script_building).target != null && other.owner.parent.getComponent(script_building).targetName == "player")) {
                        other.owner.parent.getComponent(script_building).onDamage(damage);
                    }
                }

                if (other.owner.name.indexOf("NPC") != -1) {
                    // let damage = 100 * Laya.timer.delta / 1000;
                    if (other.owner.getComponent(script_npc) != null) {
                        other.owner.getComponent(script_npc).onHit();
                    }
                }
            }

        }

        onTriggerExit(other) {
            // console.log("============离开触发器范围==============");
            if (!SceneManager.gameUI.deathFlag) {
                if (other.owner.parent.name.indexOf("building") != -1) {
                    other.owner.parent.getComponent(script_building).reSetBuilding();
                    if (SceneManager.triggerList.length == 1) {
                        SceneManager.triggerList = [];
                    } else {
                        for (var key in SceneManager.triggerList) {
                            if (SceneManager.triggerList[key].owner != null) {
                                if (SceneManager.triggerList[key].owner.parent.name == other.owner.parent.name) {
                                    SceneManager.triggerList.splice(key, 1);
                                    break;
                                }
                            } else {
                                SceneManager.triggerList.splice(key, 1);
                            }
                        }
                    }
                }
            }
            // console.log(SceneManager.triggerList);
        }
        onCollisionEnter(collision) {
        }
        onCollisionStay(collision) {
        }
        onCollisionExit(collision) {
            // console.log("============离开碰撞器范围==============");
        }
        checkAtObject(name) {
            // console.log(name);
            if (name.indexOf("npc") >= 0) {
                // console.log(name.indexOf("car"));
                let id = name.substr(name.lastIndexOf("_") + 1);
                return parseInt(id);
            }
            return -1;
        }
    }

    /**This class is automatically generated by LayaAirIDE, please do not make any modifications. */
    var View=Laya.View;
    var Dialog=Laya.Dialog;
    var Scene=Laya.Scene;
    var REG = Laya.ClassUtils.regClass;
    class gameOverUI extends Scene {
    	constructor(){ 
    		super();
    	}
    	createChildren() {
    		super.createChildren();
    		this.loadScene("View/gameOver");
    	}
    }
    REG("ui.View.gameOverUI",gameOverUI);
    class pauseViewUI extends View {
    	constructor(){ 
    		super();
    	}
    	createChildren() {
    		super.createChildren();
    		this.loadScene("View/pauseView");
    	}
    }
    REG("ui.View.pauseViewUI",pauseViewUI);
    class skinshowViewUI extends View {
    	constructor(){ 
    		super();
    	}
    	createChildren() {
    		super.createChildren();
    		this.loadScene("View/skinshowView");
    	}
    }
    REG("ui.View.skinshowViewUI",skinshowViewUI);
    class speedUPViewUI extends View {
    	constructor(){ 
    		super();
    	}
    	createChildren() {
    		super.createChildren();
    		this.loadScene("View/speedUPView");
    	}
    }
    REG("ui.View.speedUPViewUI",speedUPViewUI);
    class trySkinUI extends View {
    	constructor(){ 
    		super();
    	}
    	createChildren() {
    		super.createChildren();
    		this.loadScene("View/trySkin");
    	}
    }
    REG("ui.View.trySkinUI",trySkinUI);

    class gameOverView extends gameOverUI {

        constructor() {
            super();
        }

        onEnable() {
            this.width = Laya.stage.width;
            this.height = Laya.stage.height;
            // Laya.timer.scale = 0.0001;
            this.backBtn.on(Laya.Event.CLICK, this, this.onClickBackBtn);
            this.rankingList.vScrollBarSkin = "";
            this.rankingList.selectHandler = new Laya.Handler(this, this.onSelectStage);
            this.initList();
            if (SceneManager.rankList[0] == 10) {
                Laya.timer.once(0.5 * 1000, this, function () {
                    SceneManager.PlaySound("win");
                });
                this.winAni.play(0, false);
            }
        }

        initList() {
            this.rankingList.array = [];
            let rankingListarray = [];
            for (var key in SceneManager.rankList) {
                console.log(key);
                let obj = {};
                if (key == 0) {
                    obj['bg_img'] = { skin: "ui/first.png" };
                    obj['name_text'] = { text: SceneManager.rankList[key] == 10 ? "1  您" : "1  " + SceneManager.NPCList[SceneManager.rankList[key] - 1].getComponent(script_npc).nameStr, color: "#eaea3e" };
                } else if (key == 1) {
                    obj['bg_img'] = { skin: "ui/second.png" };
                    obj['name_text'] = { text: SceneManager.rankList[key] == 10 ? "2  您" : "2  " + SceneManager.NPCList[SceneManager.rankList[key] - 1].getComponent(script_npc).nameStr, color: "#b93dea" };
                } else if (key == 2) {
                    obj['bg_img'] = { skin: "ui/thrid.png" };
                    obj['name_text'] = { text: SceneManager.rankList[key] == 10 ? "3  您" : "3  " + SceneManager.NPCList[SceneManager.rankList[key] - 1].getComponent(script_npc).nameStr, color: "#47a6e5" };
                } else {
                    obj['bg_img'] = { skin: "ui/otherRank.png" };
                    obj['name_text'] = { text: SceneManager.rankList[key] == 10 ? (parseInt(key) + 1) + "  您" : (parseInt(key) + 1) + "  " + SceneManager.NPCList[SceneManager.rankList[key] - 1].getComponent(script_npc).nameStr, color: "#6ce845" };
                }
                rankingListarray.push(obj);
            }
            this.rankingList.array = rankingListarray;
        }
        onSelectStage(index) {
            console.log(index);
        }

        onClickBackBtn() {
            SceneManager.PlaySound("anniu");
            SceneManager.backToMain();
            this.destroy();
        }

        onDisable() {
            Laya.timer.scale = 1;
        }
    }

    class pauseView extends pauseViewUI {

        constructor() {
            super();
        }

        onEnable() {
            SceneManager.pauseUI = this;
            this.width = Laya.stage.width;
            this.height = Laya.stage.height;
            Laya.timer.scale = 0.0001;
            if (!SceneManager.gameUI) {
                this.restartBtn.visible = false;
                this.backBtn.visible = false;
            }


            this.closepauseBtn.on(Laya.Event.CLICK, this, this.onClickClosePause);
            this.kaiBtn.on(Laya.Event.CLICK, this, this.onClickkaiBtn);
            this.guanBtn.on(Laya.Event.CLICK, this, this.onClickguanBtn);
            this.restartBtn.on(Laya.Event.CLICK, this, this.onClickrestartBtn);
            this.backBtn.on(Laya.Event.CLICK, this, this.onClickBackBtn);
            if (Laya.SoundManager.soundVolume == 0) {
                this.guanBtn.getChildAt(0).visible = true;
                this.kaiBtn.getChildAt(0).visible = false;
            } else {
                this.kaiBtn.getChildAt(0).visible = true;
                this.guanBtn.getChildAt(0).visible = false;
            }

        }

        onClickBackBtn() {
            SceneManager.PlaySound("anniu");
            SceneManager.backToMain();
            this.destroy();
        }

        onClickrestartBtn() {
            SceneManager.restartGame();
            this.destroy();
        }

        onClickkaiBtn() {
            SceneManager.PlaySound("anniu");
            if (!this.kaiBtn.getChildAt(0).visible) {
                this.kaiBtn.getChildAt(0).visible = true;
                this.guanBtn.getChildAt(0).visible = false;
                Laya.SoundManager.setSoundVolume(1);
                Laya.SoundManager.setMusicVolume(1);
            }
        }

        onClickguanBtn() {
            if (!this.guanBtn.getChildAt(0).visible) {
                this.guanBtn.getChildAt(0).visible = true;
                this.kaiBtn.getChildAt(0).visible = false;
                Laya.SoundManager.setSoundVolume(0);
                Laya.SoundManager.setMusicVolume(0);
            }
        }

        onClickClosePause() {
            SceneManager.PlaySound("anniu");
            this.destroy();
        }

        onDisable() {
            Laya.timer.scale = 1;
            SceneManager.pauseUI = null;
        }
    }

    class ui_loading extends Laya.Scene {
        onEnable() {
            console.log("进入游戏场景");
            this.width = Laya.stage.width;
            this.height = Laya.stage.height;
            SceneManager.gameUI = this;
            this.playerLevel = 1;
            this.playerScale = 1;
            this.levelUPExp = 0;
            this.daojiTime = 6 * 1000;
            this.gameTime = 180 * 1000;
            this.score = 0;
            this.killNPCNum = 0;
            this.deathFlag = false;
            this.gameOverFlag = false;
            this.quaternion = new Laya.Quaternion();
            this.direction = new Laya.Vector3();
            Laya.stage.addChildAt(SceneManager.mainScene, 0);
            this.initClick();

            //创建方向光
            this.directionLight = SceneManager.mainScene.addChild(new Laya.DirectionLight());
            //设置灯光颜色
            this.directionLight.color = new Laya.Vector3(0.85, 0.85, 0.8);
            //设置光线强度
            this.directionLight.intensity = 1.2;
            // this.directionLight.transform.rotate(new Laya.Vector3(-Math.PI / 3, 0, 0));
            //设置灯光方向
            var mat = this.directionLight.transform.worldMatrix;
            mat.setForward(new Laya.Vector3(1.0, -2, 1.0));
            this.directionLight.transform.worldMatrix = mat;
            // Laya.timer.frameLoop(1, this, this.onFrameLoop);
            Laya.timer.frameLoop(1, this, this.update);

            this.directionLight.shadowMode = Laya.ShadowMode.Hard;
            this.directionLight.shadowDistance = 35;
            this.directionLight.shadowResolution = 2048;
            this.directionLight.shadowCascadesMode = Laya.ShadowCascadesMode.NoCascades;

            //这里创建的是辅助光
            //创建方向光
            this.directionLight1 = SceneManager.mainScene.addChild(new Laya.DirectionLight());
            //设置灯光颜色
            this.directionLight1.color = new Laya.Vector3(0.85, 0.85, 0.8);
            //设置光线强度
            this.directionLight1.intensity = 0.5;
            // this.directionLight.transform.rotate(new Laya.Vector3(-Math.PI / 3, 0, 0));
            //设置灯光方向
            var mat1 = this.directionLight1.transform.worldMatrix;
            mat1.setForward(new Laya.Vector3(-1.0, -2, -1.0));
            this.directionLight1.transform.worldMatrix = mat1;

            console.log(SceneManager.mainScene);
            Laya.Scene.open("Rocker.scene", false);
            let name = "BlackHole";
            let blackHole = SceneManager.model.getChildByName(name);
            SceneManager.blackHole = Laya.Sprite3D.instantiate(blackHole, SceneManager.mainScene);
            SceneManager.blackHole.transform.setWorldLossyScale(new Laya.Vector3(this.playerScale, this.playerScale, this.playerScale));
            let p = SceneManager.mainScene.getChildByName("Born" + SceneManager.RandomInt(1, 10));
            SceneManager.blackHole.transform.localPosition = new Laya.Vector3(p.transform.position.x, p.transform.position.y, p.transform.position.z);
            SceneManager.blackHole.transform.rotation = p.transform.rotation;
            SceneManager.blackHole.addComponent(TriggerCollisionScript);
            this.createBuliding();
            this.createNPC();
            this.changeKillText();
            console.log("SceneManager.skinID    " + SceneManager.skinID);
            this.showPiFu(SceneManager.skinID);
            if (SceneManager.speedUPFlag) {
                this.playerSpeedUp();
            } else {
                this.speedUpBtn.visible = true;
            }
            // Laya.timer.once(2 * 1000, this, this.playerOnHit);

            SceneManager.mainScene.getChildByName("Ground").meshRenderer.material.receiveShadow = true;
        }

        playerSpeedUp() {
            SceneManager.PlaySound("anniu");
            if (!SceneManager.speedUPFlag) {
                SceneManager.speedUPFlag = true;
            }
            this.speedUpBtn.visible = false;
            Laya.timer.clear(this, this.speedUpOver);
            Laya.timer.once(20 * 1000, this, this.speedUpOver);
        }

        speedUpOver() {
            SceneManager.speedUPFlag = false;
            this.speedUpBtn.visible = true;
        }
        initClick() {
            this.speedUpBtn.on(Laya.Event.CLICK, this, this.playerSpeedUp);
            this.pauseBtn.on(Laya.Event.CLICK, this, function () {
                SceneManager.PlaySound("anniu");
                Laya.stage.addChild(new pauseView);
            });
        }

        showPiFu(id) {
            for (var key in SceneManager.blackHole._children) {
                if (key != id && key > 0) {
                    SceneManager.blackHole._children[key].active = false;
                }
            }
        }

        onFrameLoop() {
            Laya.Quaternion.createFromYawPitchRoll(0.025 / 3, 0, 0, this.quaternion);
            this.directionLight.transform.worldMatrix.getForward(this.direction);
            Laya.Vector3.transformQuat(this.direction, this.quaternion, this.direction);
            this.directionLight.transform.worldMatrix.setForward(this.direction);
            let mat = this.directionLight.transform.worldMatrix;
            mat.setForward(this.direction);
            this.directionLight.transform.worldMatrix = mat;
        }

        playerMove(dirX, dirY, radians) {
            if (SceneManager.banMove)
                return;
            script_player.Move(dirX, dirY, radians);
        }

        createBuliding() {
            let model;
            let p;
            for (var key in SceneManager.mainScene.getChildByName("Buildings_A")._children) {
                model = SceneManager.model.getChildByName("A" + SceneManager.RandomInt(1, 12));
                p = SceneManager.mainScene.getChildByName("Buildings_A")._children[key];
                let building = Laya.Sprite3D.instantiate(model, SceneManager.mainScene, false, p.transform.position);
                let buildingJS = building.addComponent(script_building);
                SceneManager.buildingsList.push(building);
                buildingJS.init(SceneManager.buildingsList.length, "A");
            }
            for (var key in SceneManager.mainScene.getChildByName("Buildings_B")._children) {
                model = SceneManager.model.getChildByName("B" + SceneManager.RandomInt(1, 11));
                p = SceneManager.mainScene.getChildByName("Buildings_B")._children[key];
                let building = Laya.Sprite3D.instantiate(model, SceneManager.mainScene, false, p.transform.position);
                let buildingJS = building.addComponent(script_building);
                SceneManager.buildingsList.push(building);
                buildingJS.init(SceneManager.buildingsList.length, "B");
            }
            for (var key in SceneManager.mainScene.getChildByName("Props_A")._children) {
                model = SceneManager.model.getChildByName("C" + SceneManager.RandomInt(1, 15));
                p = SceneManager.mainScene.getChildByName("Props_A")._children[key];
                let building = Laya.Sprite3D.instantiate(model, SceneManager.mainScene, false, p.transform.position);
                let buildingJS = building.addComponent(script_building);
                SceneManager.buildingsList.push(building);
                buildingJS.init(SceneManager.buildingsList.length, "C");
            }
            for (var key in SceneManager.mainScene.getChildByName("Props_B")._children) {
                model = SceneManager.model.getChildByName("D" + SceneManager.RandomInt(1, 10));
                p = SceneManager.mainScene.getChildByName("Props_B")._children[key];
                let building = Laya.Sprite3D.instantiate(model, SceneManager.mainScene, false, p.transform.position);
                let buildingJS = building.addComponent(script_building);
                SceneManager.buildingsList.push(building);
                buildingJS.init(SceneManager.buildingsList.length, "D");
            }
        }

        createNPC() {
            let model;
            let p;
            for (var key = 0; key < 9; key++) {
                model = SceneManager.model.getChildByName("NPC");
                p = SceneManager.mainScene.getChildByName("Born" + SceneManager.RandomInt(1, 10));
                let NPC = Laya.Sprite3D.instantiate(model, SceneManager.mainScene, false, p.transform.position);
                let NPCJS = NPC.addComponent(script_npc);
                SceneManager.NPCList.push(NPC);
                NPCJS.init(SceneManager.NPCList.length);
            }
            this.updateRankingList();
        }

        playerOnHit() {
            this.deathFlag = true;
            this.scene_Box.gray = true;
            this._deathbg.visible = true;
            SceneManager.rockerView.turnGray();
            // SceneManager.rockerView.resetClickandUp();
            this.fuhuo_text.visible = true;
            Laya.timer.frameLoop(1, this, this.daojishi, [Laya.timer.delta]);
        }

        daojishi(dt) {
            this.daojiTime = this.daojiTime - dt;
            var sec = Math.floor(this.daojiTime / 1000 % 60);
            this.fuhuo_text.text = sec.toString();
            if (this.daojiTime <= 1100) {
                Laya.timer.clear(this, this.daojishi);
                this.daojiTime = 6 * 1000;
                this.playerRelive();
            }
        }

        playerRelive() {
            this.deathFlag = false;
            this.scene_Box.gray = false;
            this._deathbg.visible = false;
            this.fuhuo_text.visible = false;
            SceneManager.rockerView.backNormal();
        }

        update() {
            if (this.gameOverFlag)
                return;
            if (this.playerScale != this.playerLevel) {
                // console.log("改变黑洞大小");
                if (this.playerScale < this.playerLevel) {
                    this.playerScale += 0.01;
                } else {
                    this.playerScale = this.playerLevel;
                }
                SceneManager.blackHole.transform.setWorldLossyScale(new Laya.Vector3(this.playerScale, this.playerScale, this.playerScale));
            }
            this.daojishiLoop(Laya.timer.delta);
        }

        addExp(type) {
            if (this.playerLevel == 1) {
                switch (type) {
                    case "A":
                        this.score += 50;
                        SceneManager.showAddScore("+50");
                        break;
                    case "B":
                        this.score += 40;
                        SceneManager.showAddScore("+40");
                        break;
                    case "C":
                        this.score += 30;
                        SceneManager.showAddScore("+30");
                        break;
                    case "D":
                        this.score += 20;
                        SceneManager.showAddScore("+20");
                        break;
                }
                this.refreshRankList();
                this.levelUP();
                return;
            }
            switch (type) {
                case "A":
                    this.levelUPExp += 50;
                    this.score += 50;
                    SceneManager.showAddScore("+50");
                    if (this.levelUPExp >= 100) {
                        this.levelUPExp = 0;
                        this.levelUP();
                    }
                    this.refreshRankList();
                    break;
                case "B":
                    this.levelUPExp += 40;
                    this.score += 40;
                    SceneManager.showAddScore("+40");
                    if (this.levelUPExp >= 100) {
                        this.levelUPExp = 0;
                        this.levelUP();
                    }
                    this.refreshRankList();
                    break;
                case "C":
                    this.levelUPExp += 20;
                    this.score += 30;
                    SceneManager.showAddScore("+30");
                    if (this.levelUPExp >= 100) {
                        this.levelUPExp = 0;
                        this.levelUP();
                    }
                    this.refreshRankList();
                    break;
                case "D":
                    this.levelUPExp += 20;
                    this.score += 20;
                    SceneManager.showAddScore("+20");
                    if (this.levelUPExp >= 100) {
                        this.levelUPExp = 0;
                        this.levelUP();
                    }
                    this.refreshRankList();
                    break;
            }
        }

        levelUP() {
            if (this.playerLevel < 2) {
                this.playerLevel += 0.2;
            } else if (this.playerLevel < 3) {
                this.playerLevel += 0.18;
            } else {
                this.playerLevel += 0.15;
            }
            if (this.playerLevel > 4) {
                this.playerLevel = 4;
            }
            this.directionLight.shadowDistance = 35 * this.playerLevel;
        }

        refreshRankList() {
            let id;
            for (var key in SceneManager.rankList) {
                if (SceneManager.rankList[key] == 10)
                    id = key;
            }
            SceneManager.score[id] = this.score;
            let temp;
            for (let i = 0; i < 10; i++) {
                for (let j = i + 1; j < 10; j++) {
                    if (SceneManager.score[i] < SceneManager.score[j]) {
                        temp = SceneManager.score[i];
                        SceneManager.score[i] = SceneManager.score[j];
                        SceneManager.score[j] = temp;
                        temp = SceneManager.rankList[i];
                        SceneManager.rankList[i] = SceneManager.rankList[j];
                        SceneManager.rankList[j] = temp;
                    }
                }
            }
            this.updateRankingList();
        }

        updateRankingList() {
            var playerRanking;
            for (var key in SceneManager.rankList) {
                if (SceneManager.rankList[key] == 10) {
                    playerRanking = parseInt(key) + 1;
                    break;
                }
            }
            this.qita_text.align = "left";
            this.qita_text.color = "#6ce845";
            if (playerRanking == 1) {
                this.diming_text.text = " 1  " + SceneManager.score[0] + "分  " + "您";
                this.dierming_text.text = " 2  " + SceneManager.score[1] + "分  " + SceneManager.NPCList[SceneManager.rankList[1] - 1].getComponent(script_npc).nameStr;
                this.disanming_text.text = " 3  " + SceneManager.score[2] + "分  " + SceneManager.NPCList[SceneManager.rankList[2] - 1].getComponent(script_npc).nameStr;
            } else if (playerRanking == 2) {
                this.diming_text.text = " 1  " + SceneManager.score[0] + "分  " + SceneManager.NPCList[SceneManager.rankList[0] - 1].getComponent(script_npc).nameStr;
                this.dierming_text.text = " 2  " + SceneManager.score[1] + "分  " + "您";
                this.disanming_text.text = " 3  " + SceneManager.score[2] + "分  " + SceneManager.NPCList[SceneManager.rankList[2] - 1].getComponent(script_npc).nameStr;
            } else if (playerRanking == 3) {
                this.diming_text.text = " 1  " + SceneManager.score[0] + "分  " + SceneManager.NPCList[SceneManager.rankList[0] - 1].getComponent(script_npc).nameStr;
                this.dierming_text.text = " 2  " + SceneManager.score[1] + "分  " + SceneManager.NPCList[SceneManager.rankList[1] - 1].getComponent(script_npc).nameStr;
                this.disanming_text.text = " 3  " + SceneManager.score[2] + "分  " + "您";
            } else if (playerRanking == 4) {
                this.diming_text.text = " 1  " + SceneManager.score[0] + "分  " + SceneManager.NPCList[SceneManager.rankList[0] - 1].getComponent(script_npc).nameStr;
                this.dierming_text.text = " 2  " + SceneManager.score[1] + "分  " + SceneManager.NPCList[SceneManager.rankList[1] - 1].getComponent(script_npc).nameStr;
                this.disanming_text.text = " 3  " + SceneManager.score[2] + "分  " + SceneManager.NPCList[SceneManager.rankList[2] - 1].getComponent(script_npc).nameStr;
                this.qita_text.text = " " + playerRanking + "  " + SceneManager.score[playerRanking - 1] + "分  " + "您";
            } else {
                this.diming_text.text = " 1  " + SceneManager.score[0] + "分  " + SceneManager.NPCList[SceneManager.rankList[0] - 1].getComponent(script_npc).nameStr;
                this.dierming_text.text = " 2  " + SceneManager.score[1] + "分  " + SceneManager.NPCList[SceneManager.rankList[1] - 1].getComponent(script_npc).nameStr;
                this.disanming_text.text = " 3  " + SceneManager.score[2] + "分  " + SceneManager.NPCList[SceneManager.rankList[2] - 1].getComponent(script_npc).nameStr;
                this.qita_text.text = "... ";
                this.qita_text.align = "center";
                this.qita_text.color = "#c6bfbe";
                this.qita1_text.text = " " + playerRanking + "  " + SceneManager.score[playerRanking - 1] + "分  " + "您";
            }
            if (playerRanking != 1) {
                SceneManager.blackHole.getChildByName("crown").active = false;
                for (key in SceneManager.NPCList) {
                    if (SceneManager.NPCList[key].getComponent(script_npc).idx == (SceneManager.rankList[0] - 1)) {
                        SceneManager.NPCList[key].getComponent(script_npc).model.getChildByName("crown").active = true;
                    } else {
                        SceneManager.NPCList[key].getComponent(script_npc).model.getChildByName("crown").active = false;
                    }
                }
            } else {
                SceneManager.blackHole.getChildByName("crown").active = true;
                for (key in SceneManager.NPCList) {
                    SceneManager.NPCList[key].getComponent(script_npc).model.getChildByName("crown").active = false;
                }
            }
            if (playerRanking <= 3) {
                this.qita.visible = false;
            } else {
                this.qita.visible = true;
            }
            if (playerRanking <= 4) {
                this.qita1.visible = false;
            } else {
                this.qita1.visible = true;
            }
        }

        daojishiLoop(dt) {
            this.gameTime = this.gameTime - dt;
            if (this.gameTime < 0) {
                SceneManager.PlaySound("end");
                this.gameOver();
                return;
            }
            var min = Math.floor(this.gameTime / 1000 / 60);
            var sec = Math.floor(this.gameTime / 1000 % 60);
            if (sec >= 10) {
                this.shijian_text.text = + min.toString() + " : " + sec.toString();
            } else {
                this.shijian_text.text = min.toString() + " : 0" + sec.toString();
            }
        }

        gameOver() {
            this.gameOverFlag = true;
            Laya.stage.addChild(new gameOverView());
        }

        changeKillText() {
            this.killnum_text.text = "击败" + this.killNPCNum + "个对手";
        }


    }

    class DataManager {
        static Init() {
            DataManager.InitPlayerData();
            DataManager.InitTableData();
        }
        static InitTableData() {
            var config = [
                "res/jsondata/airplane.json",
                "res/jsondata/airplane_introduce.json",
            ];
            // Laya.loader.load(config, Laya.Handler.create(this, function () {
            //     DataManager.plane = Laya.loader.getRes(config[0]);
            //     DataManager.planeIntroduce = Laya.loader.getRes(config[1]);
            //     SceneManager.loadingUI.InitRes();
            // }));
            SceneManager.loadingUI.InitRes();
        }
        static InitPlayerData() {
            console.log("初始化玩家数据");
            let newDay = false;
            var s = Laya.LocalStorage.getItem("userData" + DataManager.version);
            console.log(JSON.stringify(s));
            if (s) {
                DataManager.UserData = JSON.parse(s);
                var day = new Date().getDate();
            }

            {//根据模板检测data
                for (var key in DataManager.template) {
                    if (DataManager.UserData[key] && DataManager.template[key]["day"] == false) {
                        continue;
                    }
                    DataManager.UserData[key] = DataManager.template[key].default;
                }
            }
            Laya.timer.loop(500, this, DataManager._saveTick);
        }
        static _saveTick() {
            if (DataManager._dirty == false)
                return;
            DataManager._saveData();
            DataManager._dirty = false;
        }
        static _saveData() {
            var data = JSON.stringify(DataManager.UserData);
            Laya.LocalStorage.setItem("userData" + DataManager.version, data);
        }
        static GetData(key) {
            if (DataManager.UserData[key] != null)
                return DataManager.UserData[key];
            return null;
        }
        static SetData(key, val) {
            DataManager.UserData[key] = val;
            DataManager._dirty = true;
        }
    }
    DataManager.version = "blackhole_0.0.2";
    DataManager._dirty = true;
    DataManager.UserData = {};
    DataManager.template = {
        "Sound": { "default": 1, "day": false },//声音开关
        "Volume": { "default": 0.5, "day": false },  //音量
        "Rocker": { "default": 0.5, "day": false },  //灵敏度
        "unlock": { "default": 6, "day": false },//已解锁的飞机
        "yssm": { "default": false, "day": false },//隐私声明
        "Skin": { "default": { "1": 1, "2": 1, "3": 1, "4": 1, "5": 1, "6": 1, "7": 0, "8": 0, "9": 0, "10": 0, "11": 0, "12": 0, "13": 0, "14": 0, "15": 0, "16": 0, "17": 0, "18": 0, "19": 0, "20": 0 }, "day": false },//皮肤列表
    };

    class JSBridge {
        static InitJSBridge() {
            //需要完整的类路径，注意与iOS的不同
            JSBridge.bridge = PlatformClass.createClass("demo.JSBridge");//创建脚步代理
            if (window.conch && window.conch.setOnBackPressedFunction) {
                window.conch.setOnBackPressedFunction(() => {
                });
            }
        }
        static showTips() {
            JSBridge.bridge.call("showTips");
        }
        static getLogo() {
            JSBridge.logoUrl = JSBridge.bridge.call("getLogo");
            console.log("JSBridge.logoUrl   " + JSBridge.logoUrl);

        }
        static initLabel() {
            JSBridge.bridge.call("initLabel");
        }
        static getJPGUrl_loading(index) {
            JSBridge.JPGUrl_loading = JSBridge.bridge.call("getJPGUrl_loading", index);
            console.log("JPGUrl_loading地址：" + JSBridge.JPGUrl_loading);
            return JSBridge.JPGUrl_loading;
        }

        static getJPGUrl_main(index) {
            JSBridge.logoUrl_main = JSBridge.bridge.call("getJPGUrl_main", index);
            console.log("logoUrl_main地址：" + JSBridge.logoUrl_main);
            return JSBridge.logoUrl_main;
        }

        static getJPGNum_loading() {
            return JSBridge.bridge.call("getJPGNum_loading");
        }

        static getJPGNum_main() {
            return JSBridge.bridge.call("getJPGNum_main");
        }

        static random(min, max) {
            return Math.floor(Math.random() * (max + 1 - min)) + min;
        }
    }
    JSBridge.bridge = null;
    JSBridge.logoUrl = null;
    JSBridge.JPGUrl_loading = null;
    JSBridge.logoUrl_main = null;

    class SDK_apk {
        static Init(labelName, key1, key2, key3) {//labelName：标签名   key1：套路时间   key2：间隔时间   key3：插屏开关
            LabelUtils.initLabel(labelName);
            setTimeout(() => {
                if (LabelUtils.getLabel(key3)) {
                    var tlsj = LabelUtils.getValue(key1, 60);
                    var sj = LabelUtils.getValue(key2, 30);
                    console.log("套路时间为:" + tlsj + "        间隔时间为:" + sj);
                    setTimeout(() => {
                        setInterval(() => {
                            SDK_apk.showNative();
                        }, sj * 1000);
                    }, tlsj * 1000);
                }
            }, 1000);
        }
        static showNative() {
            if (window.android) {
                window.android.showNative();
            }
        }
        static showNativeIcon() {
            if (window.android && LabelUtils.getLabel("icon")) {
                window.android.showIconAD(1);
            }
        }
        static showBanner() {
            // if (window.android) {
            //     window.android.showBanner(2);
            // }
        }
        static getLogoUrl() {
            if (window.android) {
                SDK_apk.logoUrl = window.android.getLogoUrl();
                console.log("logo地址：" + SDK_apk.logoUrl);
            }
        }
        static getJPGUrl_loading(index) {
            if (window.android) {
                SDK_apk.JPGUrl_loading = window.android.getJPGUrl_loading(index);
                console.log("JPGUrl_loading地址：" + SDK_apk.JPGUrl_loading);
                return SDK_apk.JPGUrl_loading;
            }
        }

        static getJPGUrl_main(index) {
            if (window.android) {
                SDK_apk.logoUrl_main = window.android.getJPGUrl_main(index);
                console.log("logoUrl_main地址：" + SDK_apk.logoUrl_main);
                return SDK_apk.logoUrl_main;
            }
        }

        static getJPGNum_loading() {
            if (window.android) {
                return window.android.getJPGNum_loading();
            }
        }

        static getJPGNum_main() {
            if (window.android) {
                return window.android.getJPGNum_main();
            }
        }
        static random(min, max) {
            return Math.round(Math.random() * (max - min) + min);
        }
    }
    SDK_apk.logoUrl = null;
    SDK_apk.JPGUrl_loading = null;
    SDK_apk.logoUrl_main = null;

    class ui_loading$1 extends Laya.Scene {
        onEnable() {
            console.log("进入加载界面");
            this.width = Laya.stage.width;
            this.height = Laya.stage.height;
            SceneManager.loadingUI = this;
            if (window.android) {
                var jpgIndex = SDK_apk.random(0, SDK_apk.getJPGNum_loading() - 1);
                console.log("jpgIndex:" + jpgIndex);
                var JPGbg = new Laya.Image(SDK_apk.getJPGUrl_loading(jpgIndex));
                JPGbg.zOrder = -1;
                JPGbg.centerX = 0;
                JPGbg.centerY = 0;
                this.addChild(JPGbg);
            }
            // if (Laya.Browser.onAndroid) {
            //     var jpgIndex = JSBridge.random(0, JSBridge.getJPGNum_loading() - 1);
            //     console.log("jpgIndex:" + jpgIndex);
            //     var JPGbg = new Laya.Image(JSBridge.getJPGUrl_loading(jpgIndex));
            //     JPGbg.zOrder = -1;
            //     JPGbg.centerX = 0;
            //     JPGbg.centerY = 0;
            //     this.addChild(JPGbg);
            // }
            if (SceneManager.curScene == -1) {
                DataManager.Init();
                SceneManager.PlayMusic(1);
            }
            else {
                console.log("来 进关卡----");
                this.Init3D1();
            }
        }
        InitRes() {
            this.onDataInit1();
        }
        onDataInit1() {
            this.onDataInit();
        }
        onDataInit() {
            console.log("初始化预制体模型");
            var path = "";
            path = "res/unity/LayaScene_Object_BlackHole/Conventional/Object_BlackHole.lh";
            let self = this;

            Laya.loader.create(path,
                Laya.Handler.create(this, function () {
                    console.log("没走到VIVO里");
                    let model = Laya.loader.getRes(path);
                    if (model) {
                        SceneManager.model = model;
                    }
                    self.loadOver();
                }), Laya.Handler.create(this, function (v) {
                    this.loadingbar.mask.graphics.clear();
                    this.loadingbar.mask.graphics.drawRect(0, 0, 692 * v, 60, "#ff0000");
                }));
        }
        Init3D1() {
            let self = this;
            let downUrl = "https://qygres.17666.mobi/xgmres/WeChat/humanFall/bprltdsj/";
            let urlType = "res1";
            // if (Laya.Browser.onWeiXin) {
            //     wxtil.downloadZIP1(self.loadLabel, downUrl, urlType, self.loadingbar, function () {
            //         console.log("VIVO:1-10关场景下载完成");
            //         self.onDataInit1();
            //     });
            // }
            // else {
            self.Init3D();
            // }
        }
        Init3D() {
            var path = "";
            let self = this;
            path = "res/unity/LayaScene_Scene_BlackHole/Conventional/Scene_BlackHole.ls";
            Laya.loader.create(path,
                Laya.Handler.create(this, function () {
                    SceneManager.mainScene = Laya.loader.getRes(path);
                    self.loadOver();
                }), Laya.Handler.create(this, function (v) {
                    this.loadingbar.mask.graphics.clear();
                    this.loadingbar.mask.graphics.drawRect(0, 0, 692 * v, 60, "#ff0000");
                }));
        }
        loadOver() {
            if (SceneManager.curScene == -1) {
                console.log("从加载界面进入主界面");
                // JSBridge.initLabel();
                SceneManager.open(SceneManager.UI_MAIN);
            }
            else {
                console.log("从加载界面进入游戏界面");
                Laya.timer.once(1, this, function () {
                    SceneManager.open(SceneManager.UI_GAME);
                });
            }
        }
    }

    class skinShopView extends skinshowViewUI {

        constructor() {
            super();
        }

        onEnable() {
            this.width = Laya.stage.width;
            this.height = Laya.stage.height;
            this.selectedID = -1;
            this.backBtn.on(Laya.Event.CLICK, this, this.onClickBackBtn);
            this.skinList.vScrollBarSkin = "";
            this.skinList.selectHandler = new Laya.Handler(this, this.onSelectStage);
            this.initList();

        }

        initList() {
            this.skinList.array = [];
            let skinListarray = [];
            var skinList = DataManager.GetData("Skin");
            for (var key = 0; key < 20; key++) {
                let obj = {};
                if (key <= 5) {
                    obj["ditu"] = { skin: skinList[key + 1] == 1 ? "ui/unlock_nor.png" : "ui/lock_img.png" };
                } else if (key <= 13) {
                    obj["ditu"] = { skin: skinList[key + 1] == 1 ? "ui/unlock_less.png" : "ui/lock_img.png" };
                } else {
                    obj["ditu"] = { skin: skinList[key + 1] == 1 ? "ui/unlock_best.png" : "ui/lock_img.png" };
                }
                obj["pifu"] = { skin: "skinimg/P" + parseInt(key + 1) + ".png" };
                obj["selected"] = { visible: false };
                skinListarray.push(obj);
            }
            this.skinList.array = skinListarray;
        }
        onSelectStage(index) {
            SceneManager.PlaySound("anniu");
            var skinList = DataManager.GetData("Skin");
            let obj = this.skinList.array[index];
            obj['selected']['visible'] = true;
            if (skinList[index + 1] == 0) {
                if (index <= 5) {
                    obj["ditu"] = { skin: "ui/unlock_nor.png" };
                } else if (index <= 13) {
                    obj["ditu"] = { skin: "ui/unlock_less.png" };
                } else {
                    obj["ditu"] = { skin: "ui/unlock_best.png" };
                }
                skinList[index + 1] = 1;
                DataManager.SetData("Skin", skinList);
            }
            if (this.selectedID >= 0) {
                obj = this.skinList.array[this.selectedID];
                obj['selected']['visible'] = false;
            }
            this.skinList.refresh();
            this.selectedID = index;
        }

        onClickBackBtn() {
            SceneManager.PlaySound("anniu");
            this.destroy();
        }

        onDisable() {
            Laya.timer.scale = 1;
        }
    }

    class trySkinView extends trySkinUI {

        constructor() {
            super();
        }

        onEnable() {
            this.width = Laya.stage.width;
            this.height = Laya.stage.height;
            Laya.timer.scale = 0.0001;
            this.closeBtn.on(Laya.Event.CLICK, this, this.onClickClose);
            this.useBtn.on(Laya.Event.CLICK, this, this.onClickUse);
            this.id;
            this.skinList = DataManager.GetData("Skin");
            this.showTrySkin();

        }

        showTrySkin() {
            this.id = this.createRandomID();
            this.skin_img.skin = "skinimg/P" + this.id + ".png";
        }

        createRandomID() {
            var id = SceneManager.RandomInt(1, 20);
            if (id == SceneManager.skinID || this.skinList[id] == 0) {
                return this.createRandomID();
            }
            return id;
        }

        onClickUse() {
            SceneManager.PlaySound("anniu");
            SceneManager.skinID = this.id;
            Laya.Scene.open("loading.scene");
            this.destroy();
        }

        onClickClose() {
            SceneManager.PlaySound("anniu");
            Laya.Scene.open("loading.scene");
            this.destroy();
        }

        onDisable() {
            Laya.timer.scale = 1;
        }
    }

    class speedUPView extends speedUPViewUI {

        constructor() {
            super();
        }

        onEnable() {
            this.width = Laya.stage.width;
            this.height = Laya.stage.height;
            Laya.timer.scale = 0.0001;
            this.closeBtn.on(Laya.Event.CLICK, this, this.onClickClosePause);
            this.confirmBtn.on(Laya.Event.CLICK, this, this.onClickConfirm);
        }

        onClickConfirm() {
            SceneManager.PlaySound("anniu");
            SceneManager.speedUPFlag = true;
            Laya.stage.addChild(new trySkinView());
            this.destroy();
        }

        onClickClosePause() {
            SceneManager.PlaySound("anniu");
            Laya.stage.addChild(new trySkinView());
            this.destroy();
        }

        onDisable() {
            Laya.timer.scale = 1;
        }
    }

    class ui_main extends Laya.Scene {
        onEnable() {
            console.log("进入主页");
            this.width = Laya.stage.width;
            this.height = Laya.stage.height;
            this.skinID = 1;
            this.startFlag = false;
            SceneManager.mainUI = this;
            if (window.android) {
                this.logo.skin = SDK_apk.logoUrl;
                var jpgIndex = SDK_apk.random(0, SDK_apk.getJPGNum_main() - 1);
                console.log("jpgIndex:" + jpgIndex);
                var JPGbg = new Laya.Image(SDK_apk.getJPGUrl_main(jpgIndex));
                JPGbg.zOrder = -1;
                JPGbg.centerX = 0;
                JPGbg.centerY = 0;
                this.addChild(JPGbg);
            }
            // if (Laya.Browser.onAndroid) {
            //     this.logo.skin = JSBridge.logoUrl;
            //     var jpgIndex = JSBridge.random(0, JSBridge.getJPGNum_main() - 1);
            //     console.log("jpgIndex:" + jpgIndex);
            //     var JPGbg = new Laya.Image(JSBridge.getJPGUrl_main(jpgIndex));
            //     JPGbg.zOrder = -1;
            //     JPGbg.centerX = 0;
            //     JPGbg.centerY = 0;
            //     this.addChild(JPGbg);
            // }
            this.skinList = DataManager.GetData("Skin");
            this.gamestartBtn.on(Laya.Event.CLICK, this, this.startGame);
            this.leftBtn.on(Laya.Event.CLICK, this, this.onClickLeftBtn);
            this.rightBtn.on(Laya.Event.CLICK, this, this.onClickRightBtn);
            this.tips.on(Laya.Event.CLICK, this, this.showTips);
            this.shezhiBtn.on(Laya.Event.CLICK, this, function () {
                SceneManager.PlaySound("anniu");
                Laya.stage.addChild(new pauseView());
            });
            this.skinshopBtn.on(Laya.Event.CLICK, this, function () {
                SceneManager.PlaySound("anniu");
                Laya.stage.addChild(new skinShopView());
            });


        }
        startGame() {
            if (this.startFlag)
                return;
            this.startFlag = true;
            SceneManager.PlaySound("anniu");
            Laya.stage.addChild(new speedUPView());
            // Laya.Scene.open("loading.scene");
        }

        onClickLeftBtn() {
            this.skinID--;
            if (this.skinID < 1) {
                this.skinID = 20;
            }
            if (this.skinList[this.skinID] == 0) {
                return this.onClickLeftBtn();
            }
            SceneManager.PlaySound("anniu");
            SceneManager.skinID = this.skinID;
            this.changeNameAndImg();
        }

        onClickRightBtn() {
            this.skinID++;
            if (this.skinID > 20) {
                this.skinID = 1;
            }
            if (this.skinList[this.skinID] == 0) {
                return this.onClickRightBtn();
            }
            SceneManager.PlaySound("anniu");
            SceneManager.skinID = this.skinID;
            this.changeNameAndImg();
        }

        changeNameAndImg() {
            this.skin_img.skin = "skinimg/P" + this.skinID + ".png";
        }
        showTips() {
            // JSBridge.showTips();
            if (window.android) {
                window.android.showNotice();
            }
        }
    }

    class ui_message extends Laya.Scene {
        constructor() {
            super();

        }

        onEnable() {
            console.log("onEnable");
            this._redLabel.visible = false;
            this._moneyDel.visible = false;
            this._moneyAdd.visible = false;
            SceneManager.messageUI = this;
            this.width = Laya.stage.width;
            this.height = Laya.stage.height;
        }
        onDisable() {
            console.log("onDisable");
            SceneManager.messageUI = null;
        }
        onOpened(param) {
            switch (param[0]) {
                case 1:  //通知
                    this.ShowInfo(param[1]);
                    break;
            }
        }
        ShowInfo(str) {
            var infoText = new Laya.Label();
            infoText.anchorX = 0.5;
            infoText.anchorY = 1;
            infoText.color = "#ffffff";
            infoText.fontSize = 45;
            infoText.centerX = 0;
            infoText.centerY = -200;
            infoText.text = str;
            infoText.alpha = 1;
            infoText.visible = true;
            Laya.stage.addChild(infoText);
            Laya.Tween.to(infoText, { centerY: -400, alpha: 0 }, 1500, null, Laya.Handler.create(this, function () {
                infoText.visible = false;
            }), 0, true, true);
        }
    }

    class RockerView extends Laya.View {

        constructor() {
            super();
        }

        onEnable() {
            this.curTouchId = 0;
            /***手指（鼠标）是否按下****/
            this.isDown = false;
            /***摇杆的角度****/
            this.angle = -1;
            /***摇杆的弧度****/
            this.radians = -1;
            /***是否左手遥控****/
            this.isleftControl = true;
            //鼠标按下事件监听
            Laya.stage.on(Laya.Event.MOUSE_DOWN, this, this.onMouseDown);
            //鼠标抬起事件监听
            Laya.stage.on(Laya.Event.MOUSE_UP, this, this.onMouseUp);
            //鼠标是否移除屏幕事件监听
            // this.touchRect.on(Laya.Event.MOUSE_OUT,this,onBlur);
            //控制器中心点位置初始化
            this.originPiont = new Laya.Point(this.width / 2, this.height / 2);
            //默认为控制器不显示
            this.visible = false;
            SceneManager.rockerView = this;
            Laya.timer.loop(15, this, this.tick);
        }
        offClickandUp() {
            Laya.stage.off(Laya.Event.MOUSE_DOWN, this, this.onMouseDown);
            Laya.stage.off(Laya.Event.MOUSE_UP, this, this.onMouseUp);
        }
        onClickandUp() {
            Laya.stage.on(Laya.Event.MOUSE_DOWN, this, this.onMouseDown);
            Laya.stage.on(Laya.Event.MOUSE_UP, this, this.onMouseUp);
        }
        resetClickandUp() {
            // script_player.Idle();
            this.isDown = false;
            this.visible = false;
            //移除摇杆移动事件监听
            Laya.stage.off(Laya.Event.MOUSE_MOVE, this, this.onMove);
            //修改摇杆角度与弧度为-1（代表无角度）
            this.radians = this.angle = -1;
        }
        onMouseDown(e) {
            //左右手遥控
            if (this.isleftControl) {
                //如果按下时是右边屏幕位置或已经按下鼠标，则返回
                if (e.stageY < Laya.stage.height / 10 * 6 || this.isDown) return;
            }
            else {
                //如果按下时是左边屏幕位置或已经按下鼠标，则返回
                if (e.stageY > Laya.stage.height / 10 * 6 || e.stageX < Laya.stage.width / 10 * 4 || this.isDown) return;
            }
            //记录当前按下id
            this.curTouchId = e.touchId;
            //已按下
            this.isDown = true;
            //更新摇杆到屏幕按下位置
            this.pos(Laya.stage.mouseX - (this.width / 2), Laya.stage.mouseY - (this.height / 2));
            //初始化摇杆控制点位置
            this.knob.pos(this.width / 2, this.height / 2);
            //按下后显示摇杆
            this.visible = true;
            //摇杆移动控制事件监听
            Laya.stage.on(Laya.Event.MOUSE_MOVE, this, this.onMove);
        }

        /*鼠标抬起事件回调*/
        onMouseUp(e) {
            //如果不是上次的点击id，返回（避免多点抬起，以第一次按下id为准）
            if (e.touchId != this.curTouchId) return;
            if (e.stageX < Laya.stage.width / 10 * 4 || this.isDown) {
                // script_player.Idle();
            }
            this.isDown = false;
            this.visible = false;
            //移除摇杆移动事件监听
            Laya.stage.off(Laya.Event.MOUSE_MOVE, this, this.onMove);
            //修改摇杆角度与弧度为-1（代表无角度）
            this.radians = this.angle = -1;
        }

        /*鼠标移动事件回调*/
        onMove(e) {
            //如果不是上次的点击id，返回（避免多点抬起，以第一次按下id为准）
            if (e.touchId != this.curTouchId) return;
            //将移动时的鼠标屏幕坐标转化为摇杆局部坐标
            var locationPos = this.globalToLocal(new Laya.Point(Laya.stage.mouseX, Laya.stage.mouseY), false);
            //更新摇杆控制点位置
            this.knob.pos(locationPos.x, locationPos.y);
            //更新控制点与摇杆中心点位置距离
            this.deltaX = locationPos.x - this.originPiont.x;
            this.deltaY = locationPos.y - this.originPiont.y;
            //计算控制点在摇杆中的角度
            var dx = this.deltaX * this.deltaX;
            var dy = this.deltaY * this.deltaY;
            this.angle = Math.atan2(this.deltaX, this.deltaY) * 180 / Math.PI;
            if (this.angle < 0) this.angle += 360;
            //对角度取整
            this.angle = Math.round(this.angle);
            //计算控制点在摇杆中的弧度
            this.radians = Math.PI / 180 * this.angle;
            //强制控制点与中心距离不超过80像素
            if (dx + dy >= 80 * 80) {
                //控制点在半径为80像素的位置（根据弧度变化）
                var x = Math.floor(Math.sin(this.radians) * 80 + this.originPiont.x);
                var y = Math.floor(Math.cos(this.radians) * 80 + this.originPiont.y);
                this.knob.pos(x, y);
            }
            else {
                //不超过80像素取原坐标
                this.knob.pos(locationPos.x, locationPos.y);
            }
        }

        onDisable() {
        }

        turnGray() {
            this.getChildAt(0).gray = true;
            this.knob.gray = true;
        }

        backNormal() {
            this.getChildAt(0).gray = false;
            this.knob.gray = false;
        }
        tick() {
            if (this.angle) {
                if (this.angle != -1) {
                    if (SceneManager.gameUI && SceneManager.blackHole) {
                        let x = this.knob.x - 86;
                        let y = this.knob.y - 86;
                        SceneManager.gameUI.playerMove(-x / 75, -y / 75, this.radians);
                    }
                }
            }
        }
    }

    /**This class is automatically generated by LayaAirIDE, please do not make any modifications. */

    class GameConfig {
        static init() {
            //注册Script或者Runtime引用
            let reg = Laya.ClassUtils.regClass;
    		reg("script/ui_game.js",ui_loading);
    		reg("script/ui_loading.js",ui_loading$1);
    		reg("script/ui_main.js",ui_main);
    		reg("script/ui_message.js",ui_message);
    		reg("RockerView.js",RockerView);
        }
    }
    GameConfig.width = 720;
    GameConfig.height = 1280;
    GameConfig.scaleMode ="fixedwidth";
    GameConfig.screenMode = "vertical";
    GameConfig.alignV = "top";
    GameConfig.alignH = "left";
    GameConfig.startScene = "loading.scene";
    GameConfig.sceneRoot = "";
    GameConfig.debug = false;
    GameConfig.stat = false;
    GameConfig.physicsDebug = false;
    GameConfig.exportSceneToJson = true;

    GameConfig.init();

    class Main {
    	constructor() {
    		//根据IDE设置初始化引擎		
    		SDK_apk.getLogoUrl();
    		// JSBridge.InitJSBridge();
    		// JSBridge.getLogo();
    		if (window["Laya3D"]) Laya3D.init(GameConfig.width, GameConfig.height);
    		else Laya.init(GameConfig.width, GameConfig.height, Laya["WebGL"]);
    		Laya["Physics"] && Laya["Physics"].enable();
    		Laya["DebugPanel"] && Laya["DebugPanel"].enable();
    		Laya.stage.scaleMode = GameConfig.scaleMode;
    		Laya.stage.screenMode = GameConfig.screenMode;
    		Laya.stage.alignV = GameConfig.alignV;
    		Laya.stage.alignH = GameConfig.alignH;
    		//兼容微信不支持加载scene后缀场景
    		Laya.URL.exportSceneToJson = GameConfig.exportSceneToJson;

    		//打开调试面板（通过IDE设置调试模式，或者url地址增加debug=true参数，均可打开调试面板）
    		if (GameConfig.debug || Laya.Utils.getQueryString("debug") == "true") Laya.enableDebugPanel();
    		if (GameConfig.physicsDebug && Laya["PhysicsDebugDraw"]) Laya["PhysicsDebugDraw"].enable();
    		if (GameConfig.stat) Laya.Stat.show();
    		Laya.alertGlobalError(true);

    		//激活资源版本控制，version.json由IDE发布功能自动生成，如果没有也不影响后续流程
    		Laya.ResourceVersion.enable("version.json", Laya.Handler.create(this, this.onVersionLoaded), Laya.ResourceVersion.FILENAME_VERSION);
    	}

    	onVersionLoaded() {
    		//激活大小图映射，加载小图的时候，如果发现小图在大图合集里面，则优先加载大图合集，而不是小图
    		Laya.AtlasInfoManager.enable("fileconfig.json", Laya.Handler.create(this, this.onConfigLoaded));
    	}

    	onConfigLoaded() {
    		//加载IDE指定的场景
    		GameConfig.startScene && Laya.Scene.open(GameConfig.startScene);
    	}
    }
    //激活启动类
    new Main();

}());
